//
//  SCCountDownNumberView.m
//  BaseStudyCommune
//
//  Created by GRT's iMac on 2021/4/7.
//  Copyright © 2021 enaea. All rights reserved.
//

#import "SCCountDownNumberView.h"

@interface SCCountDownNumberView ()
@property(nonatomic,strong)UILabel *label0;
@property(nonatomic,strong)UILabel *label1;
@property(nonatomic,assign)NSInteger maxNumber;

@property (nonatomic,strong) UIColor *bgColor;//背景颜色
@property (nonatomic,assign) UIFont *textFont;//字体大小
@property (nonatomic,strong) UIColor *textColor;//字体颜色
@property (nonatomic,assign) CGFloat cornerRadius;//圆角

/// 是否允许时间滚动
@property (nonatomic, assign) BOOL scrollAnimationEnabled;
@end

@implementation SCCountDownNumberView

#pragma mark - LIFE CIRCLE FUNCTIONS



#pragma mark - PUBLIC FUNCTIONS

- (instancetype)initWithFrame:(CGRect)frame maxNumber:(NSInteger)maxNumber font:(UIFont *)textFont textColor:(UIColor *)textColor bgColor:(UIColor *)bgColor cornerRadius:(CGFloat)cornerRadius scrollAnimation:(BOOL)scrollAnimation {
    if (self = [super initWithFrame:frame]) {
        //初始化赋值
        self.maxNumber = maxNumber;
        self.textFont = textFont;
        self.textColor = textColor;
        self.bgColor = bgColor;
        self.cornerRadius = cornerRadius;
        self.scrollAnimationEnabled = scrollAnimation;
        [self setupUI];
    }
    return self;
}

- (void)numberChange {
    if (self.scrollAnimationEnabled) {
        [UIView animateWithDuration:0.5 animations:^{
            self.contentOffset = CGPointMake(0, 0);
        } completion:^(BOOL finished) {
            self.currentNumber--;
            if (self.currentNumber < 0) {
                self.currentNumber = self.maxNumber;
            }
            self.label0.text = [NSString stringWithFormat:@"%ld", (long)self.currentNumber];
            self.contentOffset = CGPointMake(0, self.frame.size.height);
            NSInteger next = self.currentNumber-1;
            if (next < 0) {
                next = self.maxNumber;
            }
            self.label1.text = [NSString stringWithFormat:@"%ld",(long)next];
        }];
    } else {
        self.contentOffset = CGPointMake(0, self.frame.size.height);
        self.currentNumber--;
        if (self.currentNumber < 0) {
            self.currentNumber = self.maxNumber;
        }
        self.label0.text = [NSString stringWithFormat:@"%ld", (long)self.currentNumber];
        NSInteger next = self.currentNumber-1;
        if (next < 0) {
            next = self.maxNumber;
        }
        self.label1.text = [NSString stringWithFormat:@"%ld",(long)next];
    }
}


#pragma mark - PRIVATE FUNCTIONS

- (void)setupUI {
    //frame
    CGFloat w = self.frame.size.width;
    CGFloat h = self.frame.size.height;
    
    self.contentSize = CGSizeMake(w, h*2);
    self.contentOffset = CGPointMake(0, h);
    self.showsVerticalScrollIndicator = NO;
    self.showsHorizontalScrollIndicator = NO;
    
    self.backgroundColor = self.bgColor;
    self.layer.cornerRadius = self.cornerRadius;
    self.scrollEnabled = NO;
    
    self.label0 = [[UILabel alloc]initWithFrame:CGRectMake(0, h, w, h)];
    self.label0.textAlignment = NSTextAlignmentCenter;
    self.label0.font = self.textFont;
    self.label0.textColor = self.textColor;
    [self addSubview:self.label0];
    self.label0.text = @"0";
    
    self.label1 = [[UILabel alloc]initWithFrame:CGRectMake(0, 0, w, h)];
    self.label1.textAlignment = NSTextAlignmentCenter;
    self.label1.font = self.textFont;
    self.label1.textColor = self.textColor;
    [self addSubview:self.label1];
    self.label1.text = @"0";
}


#pragma mark - ACTION FUNCTIONS




#pragma mark - SETTER FUNCTIONS

- (void)setCurrentNumber:(NSInteger)currentNumber{
    _currentNumber = currentNumber;
    if (currentNumber <= self.maxNumber && currentNumber >= 0 ) {
        self.label0.text = [NSString stringWithFormat:@"%ld", (long)currentNumber];
        NSInteger next = currentNumber-1;
        if (next < 0) {
            next = self.maxNumber;
        }
        self.label1.text = [NSString stringWithFormat:@"%ld", (long)next];
    }
}


#pragma mark - GETTER FUNCTIONS




#pragma mark - REQUEST FUNCTIONS




#pragma mark - xxxDATASOURCE




#pragma mark - xxxDELEGATE


@end
