//
//  SCCountDownView.m
//  BaseStudyCommune
//
//  Created by GRT's iMac on 2021/4/7.
//  Copyright © 2021 enaea. All rights reserved.
//

#import "SCCountDownView.h"
#import "SCCountDownNumberView.h"

@interface SCCountDownView ()
///
@property (nonatomic, strong) SCCountDownConfig *config;
///
@property(nonatomic, assign) NSInteger number;
///
@property(nonatomic, copy) void(^change)(NSInteger);
///
@property(nonatomic, copy) void(^end)(void);
///
@property(nonatomic, strong) NSTimer *timer;
///
@property (nonatomic, strong) UILabel *dayLB;
/// 小时与分钟间的 ':'
@property (nonatomic, strong) UILabel *hourUnit;
/// 分钟与秒间的 ':'
@property (nonatomic, strong) UILabel *minUnit;
///
@property(nonatomic, strong) SCCountDownNumberView *hourH;
///
@property(nonatomic, strong) SCCountDownNumberView *hourL;
///
@property(nonatomic, strong) SCCountDownNumberView *minuteH;
///
@property(nonatomic, strong) SCCountDownNumberView *minuteL;
///
@property(nonatomic, strong) SCCountDownNumberView *secondH;
///
@property(nonatomic, strong) SCCountDownNumberView *secondL;
/// 是否正在跑
@property (nonatomic, assign) BOOL running;

@end

@implementation SCCountDownView

#pragma mark - LIFE CIRCLE FUNCTIONS

- (void)dealloc {
    if (self.timer) {
        [self.timer invalidate];
        self.timer = nil;
    }
}


#pragma mark - PUBLIC FUNCTIONS

- (instancetype)initWithFrame:(CGRect)frame config:(SCCountDownConfig *)config countDownChange:(void(^)(NSInteger time))countDownChange countDownEnd:(void(^)(void))countDownEnd {
    self = [super initWithFrame:frame];
    if (self = [super initWithFrame:frame]) {
        self.config = config;
        self.change = countDownChange;
        self.end = countDownEnd;
        self.backgroundColor = self.config.bgColor;
    }
    return self;
}

- (void)setupCurrentTime:(NSInteger)currentTime{
    if (currentTime <= 0) {
        return;
    }
    NSInteger day = currentTime/(3600*24);
    
    self.number = currentTime;
    
    currentTime -= (day*3600*24);
    NSInteger hour = currentTime/3600;
    NSInteger hourLeft = hour/10;
    NSInteger hourRight = hour%10;
    
    [self setupUIWithDay:day hourH:hourLeft hourL:hourRight];
    
    [self.hourH setCurrentNumber:hourLeft];
    [self.hourL setCurrentNumber:hourRight];
        
    currentTime -= hour*3600;
    NSInteger minute = currentTime/60;
    NSInteger minuteLeft = minute/10;
    NSInteger minuteRight = minute%10;
    [self.minuteH setCurrentNumber:minuteLeft];
    [self.minuteL setCurrentNumber:minuteRight];
    
    currentTime -= minute * 60;
    NSInteger secondLeft = currentTime/10;
    NSInteger secondRight = currentTime%10;
    [self.secondH setCurrentNumber:secondLeft];
    [self.secondL setCurrentNumber:secondRight];
    
    self.timer = [NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(numberChange) userInfo:nil repeats:YES];
}

- (void)invalidateTimer {
    if (self.timer) {
        [self.timer invalidate];
    }
}


#pragma mark - PRIVATE FUNCTIONS

-(void)setupUIWithDay:(NSInteger)day hourH:(NSInteger)hourH hourL:(NSInteger)hourL {
    self.layer.masksToBounds = YES;
    
    CGFloat width = self.frame.size.width;
    CGFloat px = width;
    CGFloat itemY = (self.frame.size.height - self.config.textLabelSize.height) /2.f;
    CGFloat itemW = self.config.textLabelSize.width;
    CGFloat height = self.config.textLabelSize.height;
    CGFloat unitW = self.config.unitSize.width; // ':' 宽度
    CGFloat unitH = self.config.unitSize.height; // ':' 高度
    CGFloat space = self.config.space; // 数字控件间隔
    CGFloat margin = self.config.margin; // 左右到边距离
    CGFloat dayW = 0.f; // 天数宽度
    if (day > 0) {
        NSString *dayStr = [NSString stringWithFormat:@"%ld天",day];
        CGSize size = [self sizeWithString:dayStr font:self.config.textFont maxSize:CGSizeMake(MAXFLOAT, height)];
        dayW = size.width;
    }
    
    px -= (102.f);
    UILabel *behandLabel = [[UILabel alloc] initWithFrame:CGRectMake(px, itemY, 102.f, height)];
    behandLabel.textAlignment = NSTextAlignmentCenter;
    behandLabel.text = _config.isSpotCheck ? @"完成抽查打卡" : @"完成课程打卡";
    behandLabel.font = self.config.textFont;
    behandLabel.textColor = self.config.textColor;
    [self addSubview:behandLabel];
    
    px -= (itemW+margin);
    self.secondL = [self getNumberViewWithOrigin:CGPointMake(px, itemY) maxNumber:9];
    [self addSubview:self.secondL];
    
    px -= (itemW + space);
    self.secondH = [self getNumberViewWithOrigin:CGPointMake(px, itemY) maxNumber:5];
    [self addSubview:self.secondH];
    
    px -= unitW;
    UILabel *minuteLabel = [[UILabel alloc] initWithFrame:CGRectMake(px, itemY, unitW, unitH)];
    minuteLabel.textAlignment = NSTextAlignmentCenter;
    minuteLabel.text = @":";
    minuteLabel.font = self.config.textFont;
    minuteLabel.textColor = self.config.textColor;
    [self addSubview:minuteLabel];
    self.minUnit = minuteLabel;
    
    px -= itemW;
    self.minuteL = [self getNumberViewWithOrigin:CGPointMake(px, itemY) maxNumber:9];
    [self addSubview:self.minuteL];
    
    px -= (itemW + space);
    self.minuteH = [self getNumberViewWithOrigin:CGPointMake(px, itemY) maxNumber:5];
    [self addSubview:self.minuteH];
    
    if (hourH > 0 || hourL > 0) {
        px -= unitW;
        UILabel *hourLabel = [[UILabel alloc]initWithFrame:CGRectMake(px, itemY, unitW, unitH)];
        hourLabel.textAlignment = NSTextAlignmentCenter;
        hourLabel.text = @":";
        hourLabel.font = self.config.textFont;
        hourLabel.textColor = self.config.textColor;
        [self addSubview:hourLabel];
        self.hourUnit = hourLabel;
        
        px -= itemW;
        self.hourL = [self getNumberViewWithOrigin:CGPointMake(px, itemY) maxNumber:((hourH == 2) ? 4 : 9)];
        [self addSubview:self.hourL];
        
        px -= (itemW + space);
        self.hourH = [self getNumberViewWithOrigin:CGPointMake(px, itemY) maxNumber:2];
        [self addSubview:self.hourH];
    }
    
    if (day > 0) {
        px -= (dayW+3);
        self.dayLB = [[UILabel alloc] initWithFrame:CGRectMake(px, itemY, dayW, height)];
        self.dayLB.textAlignment = NSTextAlignmentCenter;
        self.dayLB.text = @"0天";
        self.dayLB.font = self.config.textFont;
        self.dayLB.textColor = self.config.textColor;
        [self addSubview:self.dayLB];
    }
    px -= (34+margin);
    UILabel *frontLabel = [[UILabel alloc] initWithFrame:CGRectMake(px, itemY, 34, height)];
    frontLabel.textAlignment = NSTextAlignmentCenter;
    frontLabel.text = @"请在";
    frontLabel.font = self.config.textFont;
    frontLabel.textColor = self.config.textColor;
    [self addSubview:frontLabel];
}



#pragma mark - ACTION FUNCTIONS

- (void)numberChange {
    [UIView animateWithDuration:0 delay:0.5 options:nil animations:^{
        self.hourUnit.hidden = !self.hourUnit.hidden;
        self.minUnit.hidden = !self.minUnit.hidden;        
    } completion:nil];
    self.number--;
    if (self.number <= 0) {
        self.running = NO;
        if (self.end) {
            self.end();
        }
        [self.timer invalidate];
        self.timer = nil;
    }else{
        self.running = YES;
        if (self.change) {
            self.change(self.number);
        }
    }
    NSInteger currentTime = self.number;
    NSInteger day = currentTime/(3600*24);
    
    currentTime -= (day*3600*24);
    NSInteger hour = currentTime/3600;
    NSInteger hourLeft = hour/10;
    NSInteger hourRight = hour%10;
    //天
    self.dayLB.text = [NSString stringWithFormat:@"%ld天", day];
    
    //小时
    if (self.hourH.currentNumber != hourLeft) {
        [self.hourH numberChange];
    }
    if (self.hourL.currentNumber != hourRight) {
        [self.hourL numberChange];
    }
    
    //分钟
    currentTime -= hour*3600;
    NSInteger minute = currentTime/60;
    NSInteger minuteLeft = minute/10;
    NSInteger minuteRight = minute%10;
    if (self.minuteH.currentNumber != minuteLeft) {
        [self.minuteH numberChange];
    }
    if (self.minuteL.currentNumber != minuteRight) {
        [self.minuteL numberChange];
    }
    currentTime -= minute*60;
    NSInteger secondLeft = currentTime/10;
    NSInteger secondRight = currentTime%10;
    if (self.secondH.currentNumber != secondLeft) {
        [self.secondH numberChange];
    }
    if (self.secondL.currentNumber != secondRight) {
        [self.secondL numberChange];
    }
}


#pragma mark - SETTER FUNCTIONS




#pragma mark - GETTER FUNCTIONS

//获取字符串size
- (CGSize)sizeWithString:(NSString *)string font:(UIFont *)font maxSize:(CGSize)maxSize{
    if (!string) {
        return CGSizeZero;
    }
    NSDictionary *dict = @{NSFontAttributeName : font};
    CGSize size = [string boundingRectWithSize:maxSize options:NSStringDrawingUsesLineFragmentOrigin attributes:dict context:nil].size;
    return size;
}

- (SCCountDownNumberView *)getNumberViewWithOrigin:(CGPoint)origin maxNumber:(NSInteger)maxNumber {
    SCCountDownNumberView *numberView = [[SCCountDownNumberView alloc]initWithFrame:CGRectMake(origin.x, origin.y, self.config.textLabelSize.width, self.config.textLabelSize.height) maxNumber:maxNumber font:self.config.numFont textColor:self.config.numColor bgColor:self.config.numBgColor cornerRadius:self.config.textCornerRadius scrollAnimation:self.config.scrollAnimatedEnabled];
    return numberView;
}

- (BOOL)isRunning {
    return self.running;
}


#pragma mark - REQUEST FUNCTIONS




#pragma mark - xxxDATASOURCE




#pragma mark - xxxDELEGATE


@end
