//
//  SCTools.h
//  StudyCommune
//
//  Created by tech on 16/2/2.
//  Copyright © 2016年 tech. All rights reserved.
//

#import <UIKit/UIKit.h>

@interface SCTools : NSObject

/**
 *  提示
 *
 *  @param msg 提示信息
 *
 *  @return     alter
 */
+ (UIAlertController *)alertWithMessage:(NSString *)msg;

+ (UIAlertController *)alertWithTitle:(NSString *)title message:(NSString *)msg;

/**
 *  成功提示
 *
 *  @param msg     提示信息
 *  @param success 点击确认后回调
 *
 *  @return        alert
 */
+ (UIAlertController *)successAlertWithMessage:(NSString *)msg success:(void (^)(void))success;

/**
 *  新消息提示
 *
 *  @param titleMsg 标题
 *  @param msg      消息
 *  @param success  点击确认后回调
 *
 *  @return alert
 */
+ (UIAlertController *)newsAlertWithTitle:(NSString *)titleMsg withMsg:(NSString *)msg success:(void (^)(void))success;

/**
 *  SVProgressHUD 封装showStatus
 */
+ (void)showWithStatus:(NSString *) status;

/**
 *  SVProgressHUD 封装showSuccess
 */
+ (void)showSuccessWithStatus:(NSString *) status;

/**
 *  SVProgressHUD 封装showError
 */
+ (void)showErrorWithStatus:(NSString *) status;

/**
 *  SVProgressHUD 封装showInfo
 */
+ (void)showInfoWithStatus:(NSString*)status;

/**
 *  SVProgressHUD 封装dismiss
 */
+ (void)showDismiss;

/**
 *  手机号码格式检测
 */
+ (BOOL)isMobileNumber:(NSString *)mobileNum;

+ (BOOL)isMobileNumberSimple:(NSString *)mobileNum;

/**
 *  邮箱格式检测
 */
+ (BOOL) validateEmail:(NSString *)email;


/**
 *  根据路径计算文件大小
 */
+ (float)folderSizeAtPath:(NSString *)path;

/**
 *  根据路径清除文件
 */
+ (void)clearCacheAtPath:(NSString *)path;

/**
 *  播放器转换为格式 00:00:00
 */
+ (NSString *)formatSecondsToString:(NSInteger)seconds;

/**
 *  切换横竖屏
 *
 *  @param orientation ：UIInterfaceOrientation
 */
+ (void)forceOrientation: (UIInterfaceOrientation)orientation;

/**
 *  判断是否竖屏
 *
 *  @return 布尔值
 */
+ (BOOL)isOrientationLandscape;

/**
 *  判断字符串为空和只为空格
 */
+ (BOOL)isBlankString:(NSString *)string;

/**
 *  网络无法连接
 */
+ (BOOL)checkNetWithStatu:(NSString *)status;

//设置渐变颜色
+ (CAGradientLayer *)gradientColor:(CGRect)frame;
//设置渐变颜色
+ (CAGradientLayer *)gradientColor1:(CGRect)frame;

//清除userDefault数据
+ (void)clearAllUserDefaultsData;

+ (void)clearUserDefaultsDataForKey:(NSString *)aKey;
//通过绘制layer得到对应的image
+ (UIImage *)imageFromLayer:(CALayer *)layer;


//返回异常信息
+ (void)showErrData:(NSData *)data;

//考试保存数据返回异常信息
+ (void)showExamSaveErrData:(NSData *)data;

//考试提交数据返回异常信息
+ (void)showExamSubmitErrData:(NSData *)data;



//判断是否有emoji
//+ (NSString *)disable_emoji:(NSString *)text;
+ (BOOL )disable_emoji:(NSString *)text;


// 文字自适应高度
+ (CGFloat)labelHeight:(UILabel *)label;
+ (CGFloat)textFieldHeight:(UITextField *)tf;

// 文字自适应宽度
+ (CGFloat)labelWidth:(UILabel *)label;

// 判断是否包含表情
//+ (BOOL)stringContrainsEmoji:(NSString *)string;

// 判断刘海屏，返回YES表示是刘海屏
+ (BOOL)isNotchScreen;

// 判断是否是刘海屏，返回刘海屏底部安全距离
+ (int)isNotchScreenInsetBottom;

//转json字符串
+ (NSString *)string2JSONString:(NSString *)string;

+ (NSString *)toJsonStrWithArray:(NSArray *)arr;


#pragma mark 判断字符串是否为浮点数
+ (BOOL)isPureFloat:(NSString*)string;


#pragma mark --- 提取html字符串中的img标签url
+ (NSArray *)filterImageUrlFromHTML:(NSString *)html;
#pragma mark --- 提取html字符串中的img标签url
+ (NSString *)filterHTMLImage:(NSString *)str;
#pragma mark --- 过滤html字符串中的空格、换行符
+ (NSString *)filterHtmlMark:(NSString *)html;

//指定宽度按比例缩放
+ (UIImage *) imageCompressForWidthScale:(UIImage *)sourceImage targetWidth:(CGFloat)defineWidth;

//强制转屏
+ (void)interfaceOrientation:(UIInterfaceOrientation)orientation;

//获取AVPlayer全屏幕播放时的视图控制器
+ (UIViewController *)getAVFullScreenViewController;

#pragma mark - 当前时间戳（以毫秒为单位）
+ (NSString *)getNowTimeTimestamp;

/**
 *  AES加密
 *
 *  @param plainText 明文
 *  @param key       密钥
 *  @param iv        向量
 *
 *  @return 密文
 */
+ (NSString *)AES128Encrypt:(NSString *)plainText key:(NSString *)key iv:(NSString *)iv;
//解密
+ (NSString *)AES128Decrypt:(NSString *)plainText key:(NSString *)key iv:(NSString *)iv;
#pragma mark - DES加密
+ (NSString *) encryptUseDES:(NSString *)plainText;

#pragma mark - data转16进制字符串
+ (NSString *)hexStringFromData:(NSData *)data;

#pragma mark - 身份证号校验
+ (BOOL)validateIDCardNumber:(NSString *)value;

#pragma mark - 根据身份证号码转换成生日
+ (NSString *)birthdayStrFromIdentityCard:(NSString *)numberStr;

#pragma mark - 获取当前年份
+ (NSString *)getCurrentYear;

#pragma mark - 判断是否是山西项目
+ (BOOL )isShanXiProjectID;

#pragma mark ……Yvan…… 保留一位小数
+ (NSNumber *)formatValue:(NSString *)value;


@end
